@@ -677,7 +677,6 @@ void KImageMapEditor::setupActions()
   connect(moveLeftAction, SIGNAL(triggered(bool)),
          SLOT( slotMoveLeft() ));
   moveLeftAction->setShortcut(KShortcut(QKeySequence(Qt::Key_Left)));
-  moveLeftAction->setShortcutContext(Qt::ApplicationShortcut);
 
     moveRightAction  = new KAction(i18n("Move Right"), this);
     actionCollection()->addAction("moveright", moveRightAction );
@@ -2912,6 +2911,8 @@ void KImageMapEditor::imageUsemap() {
        it.next();
        if (it.key() != "tagname") {
            imgEl->htmlCode += " " + it.key() + "=\"";
+           if (it.key() == "usemap")
+               imgEl->htmlCode += '#';
            imgEl->htmlCode += it.value();
            imgEl->htmlCode += '"';
        }
