@@ -40,7 +40,7 @@
 
 SearchManager::SearchManager(int max_simultaneous_connections, int time_out,
                              QObject *parent)
-        : QObject(parent), recheck_mode_(false), 
+        : QObject(parent), is_login_post_request_(false), recheck_mode_(false), 
         max_simultaneous_connections_(max_simultaneous_connections), has_document_root_(false), 
         depth_(-1), current_depth_(0), external_domain_depth_(0),
         current_node_(0), current_index_(0), links_being_checked_(0),
@@ -56,6 +56,11 @@ SearchManager::SearchManager(int max_simultaneous_connections, int time_out,
 
     root_.setIsRoot(true);
 
+    if(KLSConfig::userAgent().isEmpty()) {
+        KLSConfig::setUserAgent(KProtocolManager::defaultUserAgent());
+    }
+    user_agent_ = KLSConfig::userAgent();
+
     m_weaver.setMaximumNumberOfThreads(10);
     connect(&m_weaver, SIGNAL(jobDone(ThreadWeaver::Job*)), SLOT(slotJobDone(ThreadWeaver::Job*)));
     
@@ -70,6 +75,7 @@ void SearchManager::reset()
 
     LinkStatusHelper::reset(&root_);
     cleanItems();
+    is_login_post_request_ = false;
     recheck_mode_ = false;
     recheck_links_.clear();
     links_rechecked_ = 0;
@@ -175,10 +181,28 @@ void SearchManager::recheckLinks(QList<LinkStatus*> const& linkstatus_list)
     checkVectorLinksToRecheck(recheck_links_);
 }
 
+void SearchManager::initRobotsParser(KUrl const& url)
+{
+    if(!url.protocol().startsWith("http")) {
+        return;
+    }
+
+    KUrl robotsUrl;
+    robotsUrl.setProtocol(url.protocol());
+    robotsUrl.setAuthority(url.authority());
+    robotsUrl.setFileName("robots.txt");
+    
+    robots_parser_.parseRobotFileUrl(robotsUrl);
+
+    robots_parser_.setUserAgent(user_agent_);
+}
+
 void SearchManager::startSearch(KUrl const& root, SearchMode const& modo)
 {
     kDebug(23100) <<  "SearchManager::startSearch()";
 
+    initRobotsParser(root);
+
     root_url_ = root;
     canceled_ = false;
 
@@ -277,7 +301,13 @@ void SearchManager::checkRoot()
     connect(checker, SIGNAL(transactionFinished(LinkStatus*, LinkChecker*)),
             this, SLOT(slotRootChecked(LinkStatus*, LinkChecker*)));
 
-    checker->check();
+    if(!is_login_post_request_) {
+        checker->check();
+    }
+    else {
+        checker->httpPost(post_url_,
+                          post_data_);
+    }
 }
 
 void SearchManager::slotRootChecked(LinkStatus* link, LinkChecker* checker)
@@ -820,6 +850,10 @@ bool SearchManager::checkable(KUrl const& url, LinkStatus const& link_parent) co
         if(reg_exp_.indexIn(url.url()) != -1)
             return false;
     }
+    if(localDomain(url) && !robots_parser_.canFetch(url))
+    {
+        return false;
+    }
 
     //kDebug(23100) <<  "url " << url.url() << " is checkable!";
     return true;
@@ -933,6 +967,8 @@ void SearchManager::removeHtmlParts()
 
 void SearchManager::save(QDomElement& element, LinkStatusHelper::Status status) const
 {
+    // Search options
+    
     // <url>
     QDomElement child_element = element.ownerDocument().createElement("url");
     child_element.appendChild(element.ownerDocument().createTextNode(root_.absoluteUrl().prettyUrl()));
@@ -970,15 +1006,21 @@ void SearchManager::save(QDomElement& element, LinkStatusHelper::Status status)
                 createTextNode(reg_exp_.pattern()));
     element.appendChild(child_element);
     
+    
+    // Results
+    
     child_element = element.ownerDocument().createElement("link_list");
     element.appendChild(child_element);
-    
+
     // root
     if(root_.checked() && LinkStatusHelper::hasStatus(&root_, status)) {
         LinkStatusHelper::save(&root_, child_element);
     }
     // redirections
     LinkStatus const* ls = &root_;
+    if(ls->checked() && LinkStatusHelper::hasStatus(ls, status)) {
+        LinkStatusHelper::save(ls, child_element);
+    }
     while(ls->isRedirection()) {
         ls = ls->redirection();
         if(!ls) {
@@ -1019,7 +1061,7 @@ void SearchManager::save(QDomElement& element, LinkStatusHelper::Status status)
 QString SearchManager::toXML(LinkStatusHelper::Status status) const
 {
     QDomDocument doc;
-    doc.appendChild(doc.createProcessingInstruction( "xml", 
+    doc.appendChild(doc.createProcessingInstruction( "xml",
                                        "version=\"1.0\" encoding=\"UTF-8\""));
     
     QDomElement root = doc.createElement("klinkstatus");
@@ -1030,6 +1072,66 @@ QString SearchManager::toXML(LinkStatusHelper::Status status) const
     return doc.toString(4);
 }
 
+QString SearchManager::buildSiteMapXml() const
+{
+    QDomDocument doc;
+    doc.appendChild(doc.createProcessingInstruction( "xml",
+                                                     "version=\"1.0\" encoding=\"UTF-8\""));
+                                                     
+    QDomElement root = doc.createElement("urlset");
+    root.setAttribute("xmlns", "http://www.sitemaps.org/schemas/sitemap/0.9");    
+    doc.appendChild(root);    
+
+    LinkStatus const* ls = &root_;
+    if(ls->checked() && localDomain(ls->absoluteUrl())
+        && !LinkStatusHelper::hasStatus(ls, LinkStatusHelper::undetermined)) {
+        LinkStatusHelper::buildSiteMapUrl(ls, root);
+    }
+
+    while(ls->isRedirection()) {
+        ls = ls->redirection();
+        if(!ls) {
+            break;
+        }
+        if(ls->checked() && localDomain(ls->absoluteUrl())
+            && !LinkStatusHelper::hasStatus(ls, LinkStatusHelper::undetermined))
+        {
+            LinkStatusHelper::buildSiteMapUrl(ls, root);
+        }
+    }
+    
+    for(int i = 0; i != search_results_.size(); ++i)
+    {
+        for(int j = 0; j != search_results_[i].size() ; ++j)
+        {
+            for(int l = 0; l != (search_results_[i])[j].size(); ++l)
+            {
+                LinkStatus* ls = ((search_results_[i])[j])[l];
+                
+                if(ls->checked() && localDomain(ls->absoluteUrl())
+                    && !LinkStatusHelper::hasStatus(ls, LinkStatusHelper::undetermined)) {
+                    LinkStatusHelper::buildSiteMapUrl(ls, root);
+                }
+                
+                while(ls->isRedirection()) {
+                    ls = ls->redirection();
+                    if(!ls) {
+                        break;
+                    }
+                    
+                    if(ls->checked() && localDomain(ls->absoluteUrl())
+                        && !LinkStatusHelper::hasStatus(ls, LinkStatusHelper::undetermined)) {
+                        LinkStatusHelper::buildSiteMapUrl(ls, root);
+                    }
+                }
+            }
+        }
+    }
+    
+    return doc.toString(4);
+}
+
+
 void SearchManager::slotJobDone(ThreadWeaver::Job* job)
 {
     AddLevelJob* addLevelJob = dynamic_cast<AddLevelJob*> (job);
@@ -1125,6 +1227,10 @@ BuildNodeJob::~BuildNodeJob()
 void BuildNodeJob::run()
 {
 //     kDebug(23100) << "\n\n\nBuildNodeJob::run\n\n\n";
+    if(m_searchManager->canceled_) {
+        return;
+    }
+
     m_searchManager->buildNewNode(m_linkStatus);
 }
 
@@ -1148,6 +1254,11 @@ void AddLevelJob::run()
         kDebug(23100) << "AddLevelJob::run: waiting for running jobs to finish";
         sleep(1);
     }
+    
+    if(m_searchManager->canceled_) {
+        return;
+    }
+
     m_searchManager->addLevel();
 }
 
